@@ -64,10 +64,11 @@ sub make_basic_config {
                      custom_template_path => "./custom-templates",
                      home_name            => "Home",
                      geo_handler          => 1,
+                     force_wgs84          => 1,
                    }
     );
 
-    eval { require CGI::Wiki::Search::Plucene; };
+    eval { require Wiki::Toolkit::Search::Plucene; };
     if ( $@ ) { $config->use_plucene ( 0 ) };
 	
     return $config;
@@ -138,7 +139,7 @@ sub write_data {
 
 =head1 AUTHOR
 
-The OpenGuides Project (openguides-dev@openguides.org)
+The OpenGuides Project (openguides-dev@lists.openguides.org)
 
 =head1 COPYRIGHT
 
